"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compatibilityToString = exports.nextVersion = exports.diffCompatibility = exports.compareConfigObjects = exports.compareSchema = exports.compareSchemas = exports.comparePackages = exports.leastCompatible = exports.DifferenceType = exports.Compability = void 0;
const semver_1 = require("semver");
var Compability;
(function (Compability) {
    Compability[Compability["Identical"] = 0] = "Identical";
    Compability[Compability["MinorChange"] = 1] = "MinorChange";
    Compability[Compability["CompatibleChange"] = 2] = "CompatibleChange";
    Compability[Compability["BreakingChange"] = 3] = "BreakingChange";
})(Compability = exports.Compability || (exports.Compability = {}));
var DifferenceType;
(function (DifferenceType) {
    DifferenceType["REMOVE_SCHEMA"] = "REMOVE_SCHEMA";
    DifferenceType["ADD_SCHEMA"] = "ADD_SCHEMA";
    DifferenceType["CHANGE_PACKAGE_DISPLAY_NAME"] = "CHANGE_PACKAGE_DISPLAY_NAME";
    DifferenceType["CHANGE_PACKAGE_DESCRIPTION"] = "CHANGE_PACKAGE_DESCRIPTION";
    DifferenceType["CHANGE_SOURCE"] = "CHANGE_SOURCE";
    DifferenceType["CHANGE_PARSER"] = "CHANGE_PARSER";
    DifferenceType["ADD_PROPERTY"] = "ADD_PROPERTY";
    DifferenceType["REMOVE_PROPERTY"] = "REMOVE_PROPERTY";
    DifferenceType["CHANGE_PROPERTY_TYPE"] = "CHANGE_PROPERTY_TYPE";
    DifferenceType["CHANGE_PROPERTY_FORMAT"] = "CHANGE_PROPERTY_FORMAT";
    DifferenceType["CHANGE_PROPERTY_DESCRIPTION"] = "CHANGE_PROPERTY_DESCRIPTION";
    DifferenceType["CHANGE_GENERATED_BY"] = "CHANGE_GENERATED_BY";
    DifferenceType["CHANGE_UPDATED_DATE"] = "CHANGE_UPDATED_DATE";
    DifferenceType["CHANGE_VERSION"] = "CHANGE_VERSION";
})(DifferenceType = exports.DifferenceType || (exports.DifferenceType = {}));
function leastCompatible(a, b) {
    if (a == b)
        return a;
    if (a < b)
        return b;
    return a;
}
exports.leastCompatible = leastCompatible;
function u(condition, newValue, existingValue) {
    if (!condition) {
        return leastCompatible(newValue, existingValue);
    }
    return Compability.BreakingChange;
}
function comparePackages(packageA, packageB) {
    let response = [];
    if (packageA.description != packageB.description)
        response.push({ type: DifferenceType.CHANGE_PACKAGE_DESCRIPTION, pointer: '#' });
    if (packageA.displayName != packageB.displayName)
        response.push({ type: DifferenceType.CHANGE_PACKAGE_DISPLAY_NAME, pointer: '#' });
    if (packageA.generatedBy != packageB.generatedBy)
        response.push({ type: DifferenceType.CHANGE_GENERATED_BY, pointer: '#' });
    if (packageA.updatedDate != packageA.updatedDate)
        response.push({ type: DifferenceType.CHANGE_UPDATED_DATE, pointer: '#' });
    response = response.concat(compareSchemas(packageA.schemas, packageB.schemas));
    return response;
}
exports.comparePackages = comparePackages;
function compareSchemas(priorSchemas, newSchemas) {
    let response = [];
    for (const schemaA of priorSchemas) {
        let found = false;
        for (const schemaB of newSchemas) {
            if (schemaA.title != schemaB.title)
                continue;
            found = true;
            response = response.concat(compareSchema(schemaA, schemaB, '#'));
        }
        if (!found)
            response.push({
                type: DifferenceType.REMOVE_SCHEMA,
                pointer: '',
            });
    }
    return response;
}
exports.compareSchemas = compareSchemas;
function compareSchema(priorSchema, newSchema, pointer = '#') {
    let response = [];
    pointer += '/' + newSchema.title;
    if (priorSchema.description != newSchema.description)
        response.push({ type: DifferenceType.CHANGE_PROPERTY_DESCRIPTION, pointer });
    if (Array.isArray(priorSchema.type) && Array.isArray(newSchema.type)) {
        if (!priorSchema.type.every((v) => { var _a; return ((_a = newSchema.type) === null || _a === void 0 ? void 0 : _a.indexOf(v)) != -1; }))
            response.push({ type: DifferenceType.CHANGE_PROPERTY_TYPE, pointer });
    }
    else if (!Array.isArray(priorSchema.type) && !Array.isArray(newSchema.type)) {
        if (priorSchema.type != newSchema.type)
            response.push({ type: DifferenceType.CHANGE_PROPERTY_TYPE, pointer });
    }
    else {
        response.push({ type: DifferenceType.CHANGE_PROPERTY_TYPE, pointer });
    }
    if (priorSchema.type == 'string' && priorSchema.format != newSchema.format)
        response.push({ type: DifferenceType.CHANGE_PROPERTY_FORMAT, pointer });
    if (priorSchema.type == 'object') {
        if (priorSchema.properties == null)
            throw new Error('Prior Schema property type is object, but has no properties');
        if (newSchema.properties == null)
            throw new Error('New Schema property type is object, but has no properties');
        for (const priorKey of Object.keys(priorSchema.properties)) {
            const propertyPointer = pointer + '/properties';
            const newKeys = Object.keys(newSchema.properties);
            if (newKeys.indexOf(priorKey) == -1) {
                response.push({ type: DifferenceType.REMOVE_PROPERTY, pointer: pointer });
                break;
            }
            const priorProperty = priorSchema.properties[priorKey];
            const newProperty = newSchema.properties[priorKey];
            response = response.concat(compareSchema(priorProperty, newProperty, propertyPointer));
        }
        for (const newKey of Object.keys(newSchema.properties)) {
            const priorKeys = Object.keys(priorSchema.properties);
            const propertyPointer = pointer + '/properties/' + newKey;
            if (priorKeys.indexOf(newKey) == -1) {
                response.push({ type: DifferenceType.ADD_PROPERTY, pointer: propertyPointer });
            }
        }
    }
    if (priorSchema.source == null && newSchema.source != null) {
        response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
    }
    else if (newSchema.source == null && priorSchema.source != null) {
        response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
    }
    else if (priorSchema.source != null && newSchema.source != null) {
        if (priorSchema.source.protocol != newSchema.source.protocol) {
            response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
        }
        else {
            const configComparison = compareConfigObjects(priorSchema.source.configuration, newSchema.source.configuration);
            if (!configComparison)
                response.push({ type: DifferenceType.CHANGE_SOURCE, pointer: pointer });
        }
    }
    if (priorSchema.parser == null && newSchema.parser != null) {
        response.push({ type: DifferenceType.CHANGE_PARSER, pointer: pointer });
    }
    else if (newSchema.parser == null && priorSchema.parser != null) {
        response.push({ type: DifferenceType.CHANGE_PARSER, pointer: pointer });
    }
    else if (priorSchema.parser != null && newSchema.parser != null) {
        if (priorSchema.parser.configuration != newSchema.parser.configuration) {
            response.push({ type: DifferenceType.CHANGE_PARSER, pointer: pointer });
        }
        else {
            const configComparison = compareConfigObjects(priorSchema.parser.configuration, newSchema.parser.configuration);
            if (!configComparison)
                response.push({ type: DifferenceType.CHANGE_PARSER, pointer: pointer });
        }
    }
    return response;
}
exports.compareSchema = compareSchema;
function compareConfigObjects(priorObject, newObject) {
    if (priorObject == null && newObject == null)
        return true;
    const newKeys = Object.keys(newObject);
    for (const priorKey of Object.keys(priorObject)) {
        if (newKeys.indexOf(priorKey) == -1)
            return false;
        const priorValue = priorObject[priorKey];
        const newValue = newObject[priorKey];
        if (typeof priorValue != typeof newValue)
            return false;
        if (typeof priorValue == 'object')
            if (!compareConfigObjects(priorValue, newValue))
                return false;
            else if (priorValue != newValue)
                return false;
    }
    return true;
}
exports.compareConfigObjects = compareConfigObjects;
function diffCompatibility(diffs) {
    let returnValue = Compability.Identical;
    diffs.forEach((d) => {
        switch (d.type) {
            case DifferenceType.REMOVE_PROPERTY:
            case DifferenceType.REMOVE_SCHEMA:
            case DifferenceType.CHANGE_PROPERTY_FORMAT:
            case DifferenceType.CHANGE_PROPERTY_TYPE:
                returnValue = Compability.BreakingChange;
                break;
            case DifferenceType.ADD_PROPERTY:
            case DifferenceType.ADD_SCHEMA:
                returnValue = Math.max(returnValue, Compability.CompatibleChange);
                break;
            case DifferenceType.CHANGE_PACKAGE_DESCRIPTION:
            case DifferenceType.CHANGE_PACKAGE_DISPLAY_NAME:
            case DifferenceType.CHANGE_PARSER:
            case DifferenceType.CHANGE_PROPERTY_DESCRIPTION:
            case DifferenceType.CHANGE_SOURCE:
                returnValue = Math.max(returnValue, Compability.MinorChange);
                break;
            case DifferenceType.CHANGE_VERSION:
            case DifferenceType.CHANGE_GENERATED_BY:
            case DifferenceType.CHANGE_UPDATED_DATE:
                break;
            default:
                throw new Error('Diff type ' + d.type + ' not mapped');
        }
    });
    return returnValue;
}
exports.diffCompatibility = diffCompatibility;
function nextVersion(currentVersion, diffCompatibility) {
    const copy = new semver_1.SemVer(currentVersion.version);
    switch (diffCompatibility) {
        case Compability.BreakingChange:
            return copy.inc('major');
        case Compability.CompatibleChange:
            return copy.inc('minor');
        case Compability.MinorChange:
            return copy.inc('patch');
        case Compability.Identical:
            return copy;
        default:
            throw new Error('Unrecognized compability ' + diffCompatibility);
    }
}
exports.nextVersion = nextVersion;
function compatibilityToString(compatibility) {
    switch (compatibility) {
        case Compability.BreakingChange:
            return 'breaking';
        case Compability.CompatibleChange:
            return 'compatibile';
        case Compability.MinorChange:
            return 'minor';
        case Compability.Identical:
            return 'no';
        default:
            throw new Error('Compatibility ' + compatibility + ' not recognized');
    }
}
exports.compatibilityToString = compatibilityToString;
//# sourceMappingURL=PackageUtil.js.map