import { JSONSchema7, JSONSchema7TypeName } from 'json-schema';
export interface RegistryReference {
    hostname: string;
    port: number;
    catalogSlug: string;
    publishData: boolean;
}
export interface PackageFile {
    packageSlug: string;
    displayName: string;
    description: string;
    schemas: Schema[];
    version: string;
    generatedBy: string;
    updatedDate: Date;
    registries?: RegistryReference[];
}
export interface Schema extends JSONSchema7 {
    source?: Source;
    parser?: ParserInfo;
    properties?: {
        [key: string]: Schema;
    };
    recordCount?: number;
    recordsNotPresent?: number;
    recordCountApproximate?: boolean;
    byteCount?: number;
    byteCountApproximate?: boolean;
    valueTypes?: {
        [key: string]: ValueTypeStatistics;
    };
}
export interface ParserInfo {
    mimeType: string;
    configuration?: any;
}
export interface Source {
    protocol: Protocol;
    configuration?: any;
}
export interface ValueTypeStatistics {
    valueType: JSONSchema7TypeName | 'date';
    recordCount?: number;
    stringMaxLength?: number;
    stringMinLength?: number;
    numberMaxValue?: number;
    numberMinValue?: number;
    dateMaxValue?: Date;
    dateMinValue?: Date;
    stringOptions?: {
        [key: string]: number;
    };
}
export declare enum Protocol {
    Http = "HTTP",
    LocalFile = "LOCAL_FILE"
}
export * from './PackageUtil';
