"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PackageUtil_1 = require("../src/PackageUtil");
const semver_1 = require("semver");
describe("Checking VersionUtil", () => {
    test("Compatibility ENUM Order", () => {
        expect(PackageUtil_1.leastCompatible(PackageUtil_1.Compability.Identical, PackageUtil_1.Compability.BreakingChange)).toEqual(PackageUtil_1.Compability.BreakingChange);
    });
    test("No change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = PackageUtil_1.nextVersion(oldVersion, PackageUtil_1.Compability.Identical);
        expect(oldVersion.version).toEqual("1.0.3");
        expect(newVersion.version).toEqual("1.0.3");
    });
    test("Minor change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = PackageUtil_1.nextVersion(oldVersion, PackageUtil_1.Compability.MinorChange);
        expect(oldVersion.version).toEqual("1.0.3");
        expect(newVersion.version).toEqual("1.0.4");
    });
    test("Compatible change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = PackageUtil_1.nextVersion(oldVersion, PackageUtil_1.Compability.CompatibleChange);
        expect(oldVersion.version).toEqual("1.0.3");
        expect(newVersion.version).toEqual("1.1.0");
    });
    test("Breaking change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = PackageUtil_1.nextVersion(oldVersion, PackageUtil_1.Compability.BreakingChange);
        expect(oldVersion.version).toEqual("1.0.3");
        expect(newVersion.version).toEqual("2.0.0");
    });
    test("Simple property schema comparison", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "string",
            format: 'date-time'
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "string",
            format: 'date-time'
        };
        expect(PackageUtil_1.compareSchema(schemaA1, schemaA2).length).toEqual(0);
        schemaA2.format = "date";
        const changeTitle = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(changeTitle[0].type).toEqual(PackageUtil_1.DifferenceType.CHANGE_PROPERTY_FORMAT);
    });
    test("Type arrays vs singluar values", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "object",
            properties: {
                "string": { title: "string", type: "string" },
                "number": { title: "number", type: 'number' }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "object",
            properties: {
                "string": { title: "string", type: "string" },
                "number": { title: "number", type: 'number' }
            }
        };
        const diff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(diff.length).toEqual(0);
        schemaA1.properties["string"].type = ['string', 'number'];
        const arrayVsNotDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(arrayVsNotDiff.length).toEqual(1);
        expect(arrayVsNotDiff[0].type).toEqual(PackageUtil_1.DifferenceType.CHANGE_PROPERTY_TYPE);
        schemaA2.properties["string"].type = ['string', 'number'];
        const equalDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(equalDiff.length).toEqual(0);
    });
    test("Object schema comparison", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "object",
            properties: {
                "string": { title: "string", type: "string" },
                "number": { title: "number", type: 'number' }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "object",
            properties: {
                "string": { title: "string", type: "string" },
                "number": { title: "number", type: 'number' }
            }
        };
        const diff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(diff.length).toEqual(0);
        schemaA2.properties["boolean"] = { title: "boolean", type: 'boolean' };
        const compatibleChange = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(compatibleChange.length).toEqual(1);
        expect(compatibleChange[0].type).toEqual(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        schemaA1.properties["date"] = { title: "date", type: 'string', format: 'date' };
        const removePropertyDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(removePropertyDiff.length).toEqual(2);
        const propertyRemoved = removePropertyDiff.find(d => d.type == PackageUtil_1.DifferenceType.ADD_PROPERTY);
        expect(propertyRemoved != null).toEqual(true);
        schemaA1.properties["boolean"] = { title: "boolean", type: 'boolean' };
        schemaA2.properties["date"] = { title: "date", type: 'string', format: 'date' };
        const finalDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(finalDiff).toHaveLength(0);
    });
    test("Diff compatibility testing", () => {
        expect(PackageUtil_1.nextVersion(new semver_1.SemVer("1.0.2"), PackageUtil_1.Compability.BreakingChange)).toEqual(new semver_1.SemVer("2.0.0"));
        expect(PackageUtil_1.nextVersion(new semver_1.SemVer("1.0.3"), PackageUtil_1.Compability.CompatibleChange)).toEqual(new semver_1.SemVer("1.1.0"));
        expect(PackageUtil_1.nextVersion(new semver_1.SemVer("1.0.3"), PackageUtil_1.Compability.MinorChange)).toEqual(new semver_1.SemVer("1.0.4"));
    });
    test("Nested Objects Comparison", () => {
        const schemaA1 = {
            title: "SchemaA",
            type: "object",
            properties: {
                "object": { title: "object", type: "object",
                    properties: {
                        "string1": { type: 'string' }
                    } },
                "number": { title: "number", type: 'number' }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            type: "object",
            properties: {
                "object": { title: "object", type: "object",
                    properties: {
                        "string1": { type: 'string' }
                    } },
                "number": { title: "number", type: 'number' }
            }
        };
        const firstDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(firstDiff).toHaveLength(0);
        expect(PackageUtil_1.diffCompatibility(firstDiff)).toEqual(PackageUtil_1.Compability.Identical);
        schemaA2.properties["object"].properties["string2"] = { type: 'string' };
        const compatibleDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(compatibleDiff).toHaveLength(1);
        expect(compatibleDiff[0].type).toEqual(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        expect(compatibleDiff[0].pointer).toEqual("#/SchemaA/properties/object/properties/string2");
        const compatibleComparison = PackageUtil_1.diffCompatibility(compatibleDiff);
        expect(compatibleComparison).toEqual(PackageUtil_1.Compability.CompatibleChange);
        schemaA1.properties["object"].properties["string3"] = { type: 'string' };
        const breakingDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(breakingDiff).toHaveLength(2);
        expect(breakingDiff[0].type).toEqual(PackageUtil_1.DifferenceType.REMOVE_PROPERTY);
        expect(breakingDiff[0].pointer).toEqual("#/SchemaA/properties/object");
        expect(breakingDiff[1].type).toEqual(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        expect(breakingDiff[1].pointer).toEqual("#/SchemaA/properties/object/properties/string2");
        const breakingChange = PackageUtil_1.diffCompatibility(breakingDiff);
        expect(breakingChange).toEqual(PackageUtil_1.Compability.BreakingChange);
        schemaA1.properties["object"].properties["string2"] = { type: 'string' };
        schemaA2.properties["object"].properties["string3"] = { type: 'string' };
        const finalDiff = PackageUtil_1.compareSchema(schemaA1, schemaA2);
        expect(finalDiff).toHaveLength(0);
        expect(PackageUtil_1.diffCompatibility(finalDiff)).toEqual(PackageUtil_1.Compability.Identical);
    });
});
//# sourceMappingURL=PackageComparison.spec.js.map